import { integer, pgTable, serial } from "drizzle-orm/pg-core";
import userTable from "../schema/user.schema";
import collectionTable from "../schema/collection.schema";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";
import { relations } from "drizzle-orm";

const collectionLikesTable = pgTable("collection_likes", {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  user_id: integer("user_id")
    .notNull()
    .references(() => userTable.id, {onDelete: 'cascade'}),
  collection_id: integer("collection")
    .notNull()
    .references(() => collectionTable.id, {onDelete: 'cascade'})
})

export const collectionLikesRelations = relations(collectionLikesTable, ({one}) => ({
    collection: one(collectionTable, {
        fields: [collectionLikesTable.collection_id],
        references: [collectionTable.id]
    }),
    user: one(userTable, {
        fields: [collectionLikesTable.user_id],
        references: [userTable.id]
    })
}))

const collectionLikesModelSchema = createSelectSchema(collectionLikesTable)
const collectionLikesDtoSchema = collectionLikesModelSchema
const collectionLikesInputSchema = createInsertSchema(collectionLikesTable)
const collectionLikesUpdateSchema = collectionLikesInputSchema.partial().required({ user_id: true, collection_id: true })

export type CollectionLikesModel = z.infer<typeof collectionLikesModelSchema>
export type CollectionLikesDto = z.infer<typeof collectionLikesDtoSchema>
export type CollectionLikesInput = z.infer<typeof collectionLikesInputSchema>
export type CollectionLikesUpdate = z.infer<typeof collectionLikesUpdateSchema>

export const collectionLikesSchemas = {
    collectionLikesModelSchema,
    collectionLikesDtoSchema,
    collectionLikesInputSchema,
    collectionLikesUpdateSchema,
}

export default collectionLikesTable
