import { integer, pgTable, serial } from "drizzle-orm/pg-core"
import { collectionTable, resourceTable } from "../schema"
import { relations } from "drizzle-orm"
import { createInsertSchema, createSelectSchema } from "drizzle-zod"
import type { z } from "zod"

const collectionResourcesTable = pgTable("collection_resources", {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  collection_id: integer("collection_id")
    .notNull()
    .references(() => collectionTable.id, { onDelete: 'cascade' }),
  resource_id: integer("resource_id")
    .notNull()
    .references(() => resourceTable.id, { onDelete: 'cascade'})
})

export const collectionResourcesRelations = relations(collectionResourcesTable, ({one}) => ({
  collection: one(collectionTable, {
    fields: [collectionResourcesTable.collection_id],
    references: [collectionTable.id]
  }),
  resource: one(resourceTable, {
    fields: [collectionResourcesTable.resource_id],
    references: [resourceTable.id]
  })
}))

const collectionResourcesModelSchema = createSelectSchema(collectionResourcesTable)
const collectionResourcesDtoSchema = collectionResourcesModelSchema
const collectionResourcesInputSchema = createInsertSchema(collectionResourcesTable)
const collectionResourcesUpdateSchema = collectionResourcesInputSchema.partial().required( { collection_id: true, resource_id: true})

export type CollectionResourcesModel = z.infer<typeof collectionResourcesModelSchema>
export type CollectionResourcesDto = z.infer<typeof collectionResourcesDtoSchema>
export type CollectionResourcesInput = z.infer<typeof collectionResourcesInputSchema>
export type CollectionResourcesUpdate = z.infer<typeof collectionResourcesUpdateSchema>

export const collectionResourcesSchemas = {
  collectionResourcesModelSchema,
  collectionResourcesDtoSchema,
  collectionResourcesInputSchema,
  collectionResourcesUpdateSchema
}

export default collectionResourcesTable