import { integer, pgTable, serial } from "drizzle-orm/pg-core";
import resourceTable from "../schema/resource.schema";
import educationalStageTable from "../schema/educational-stage.schema";
import { relations } from "drizzle-orm";
import { createSelectSchema } from "drizzle-zod";
import type { z } from "zod";


const resourceEducationalStagesTable = pgTable('resource_educational_stages', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  resource_id: integer('resource_id')
    .notNull()
    .references(() => resourceTable.id, { onDelete: 'cascade' }),
  educational_stage_id: integer('educational_stage_id')
    .notNull()
    .references(() => educationalStageTable.id, { onDelete: 'cascade' })
})

export const resourceEducationalStagesRelations = relations(resourceEducationalStagesTable, ({ one }) => ({
    resource: one(resourceTable, {
        fields: [resourceEducationalStagesTable.resource_id],
        references: [resourceTable.id]
    }),
    educationalStage: one(educationalStageTable, {
        fields: [resourceEducationalStagesTable.educational_stage_id],
        references: [educationalStageTable.id]
    })
}))

const resourceEducationalStagesModelSchema = createSelectSchema(resourceEducationalStagesTable)
const resourceEducationalStagesDtoSchema = resourceEducationalStagesModelSchema.omit({})
const resourceEducationalStagesInputSchema = createSelectSchema(resourceEducationalStagesTable)
const resourceEducationalStagesUpdateSchema = resourceEducationalStagesInputSchema

export type ResourceEducationalStagesModel = z.infer<typeof resourceEducationalStagesModelSchema>
export type ResourceEducationalStagesDto = z.infer<typeof resourceEducationalStagesDtoSchema>
export type ResourceEducationalStagesInput = z.infer<typeof resourceEducationalStagesInputSchema>
export type ResourceEducationalStagesUpdate = z.infer<typeof resourceEducationalStagesUpdateSchema>

export const resourceEducationalStagesSchema = {
    model: resourceEducationalStagesModelSchema,
    dto: resourceEducationalStagesDtoSchema,
    input: resourceEducationalStagesInputSchema,
    update: resourceEducationalStagesUpdateSchema,
}

export default resourceEducationalStagesTable