import { integer, pgTable, serial } from "drizzle-orm/pg-core";
import resourceTable from "../schema/resource.schema";
import languageTable from "../schema/language.schema";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";
import { relations } from "drizzle-orm";


const resourceLanguagesTable = pgTable('resource_languages', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  resource_id: integer('resource_id')
    .notNull()
    .references(() => resourceTable.id, {onDelete:'cascade'}),
  language_id: integer('language_id')
    .notNull()
    .references(() => languageTable.id, {onDelete:'cascade'})
})


export const resourceLanguagesRelations = relations(resourceLanguagesTable, ({one}) => ({
    resource: one(resourceTable, {
        fields: [resourceLanguagesTable.resource_id],
        references: [resourceTable.id]
    }),
    language: one(languageTable, {
        fields: [resourceLanguagesTable.language_id],
        references: [languageTable.id]
    })
}))



const resourceLanguagesModelSchema = createSelectSchema(resourceLanguagesTable)
const resourceLanguagesDtoSchema = resourceLanguagesModelSchema.omit({})
const resourceLanguagesInputSchema = createInsertSchema(resourceLanguagesTable)
const resourceLanguagesUpdateSchema = resourceLanguagesInputSchema

export type ResourceLanguagesModel = z.infer<typeof resourceLanguagesModelSchema>
export type ResourceLanguagesDto = z.infer<typeof resourceLanguagesDtoSchema>
export type ResourceLanguagesInput = z.infer<typeof resourceLanguagesInputSchema>
export type ResourceLanguagesUpdate = z.infer<typeof resourceLanguagesUpdateSchema>


export const resourceLanguagesSchema = {
    model: resourceLanguagesModelSchema,
    dto: resourceLanguagesDtoSchema,
    input: resourceLanguagesInputSchema,
    update: resourceLanguagesUpdateSchema,
}

export default resourceLanguagesTable