import { integer, pgTable, serial } from "drizzle-orm/pg-core";
import { achievementTable, userTable } from "../schema";
import { relations } from "drizzle-orm";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";

const userAchievementsTable = pgTable("user_achievements", {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  user_id: integer("user_id")
    .notNull()
    .references(() => userTable.id, {onDelete: 'cascade'}),
  achievement_id: integer("achievement_id")
    .notNull()
    .references(() => userTable.id, {onDelete: 'cascade'})
})

export const userAchievementRelations = relations(userAchievementsTable, ({one}) => ({
    user: one (userTable, {
        fields: [userAchievementsTable.user_id],
        references: [userTable.id]
    }),
    achievement: one(achievementTable, {
        fields: [userAchievementsTable.achievement_id],
        references: [achievementTable.id]
    })
}))

const userAchievementsModelSchema = createSelectSchema(userAchievementsTable)
const userAchievementsRelationsDtoSchema = userAchievementsModelSchema
const userAchievementsInputSchema = createInsertSchema(userAchievementsTable)
const userAchievementsUpdateSchema = userAchievementsInputSchema.partial().required({ user_id: true, achievement_id: true })

export type UserAchievementsModel = z.infer<typeof userAchievementsModelSchema>
export type UserAchievementDto = z.infer<typeof userAchievementsRelationsDtoSchema>
export type UserAchievementsInput = z.infer<typeof userAchievementsInputSchema>
export type UserAchievementsUpdate = z.infer<typeof userAchievementsUpdateSchema>

export const userAchievementsSchemas = {
    userAchievementsModelSchema,
    userAchievementsRelationsDtoSchema,
    userAchievementsInputSchema,
    userAchievementsUpdateSchema,
}

export default userAchievementsTable