import { serial, pgTable, integer } from "drizzle-orm/pg-core"
import { roleTable, userTable } from "../schema"
import { relations } from "drizzle-orm"
import { createInsertSchema, createSelectSchema } from "drizzle-zod"
import type { z } from "zod"

const userRoleRelationTable = pgTable('user_roles', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  user_id: integer('user_id')
    .references(() => userTable.id, {onDelete: 'cascade'})
    .notNull(),
  role_id: integer('role_id')
    .references(() => roleTable.id, {onDelete: 'cascade'})
    .notNull(),
})

export const userRoleRelationTableRelations = relations(
  userRoleRelationTable,
  ({ one }) => ({
    user: one(userTable, {
      fields: [userRoleRelationTable.user_id],
      references: [userTable.id]
    }),
    role: one(roleTable, {
      fields: [userRoleRelationTable.role_id],
      references: [roleTable.id]
    }),
  })
)

const userRoleRelationModelSchema = createSelectSchema(userRoleRelationTable)
const userRoleRelationDtoSchema = userRoleRelationModelSchema
const userRoleRelationInputSchema = createInsertSchema(userRoleRelationTable)
const userRoleRelationUpdateSchema = userRoleRelationInputSchema
  .partial()
  .required({id: true})

export type UserRoleRelationModel = z.infer<typeof userRoleRelationModelSchema>
export type UserRoleRelationDto = z.infer<typeof userRoleRelationDtoSchema>
export type UserRoleRelationInput = z.infer<typeof userRoleRelationInputSchema>
export type UserRoleRelationUpdate = z.infer<typeof userRoleRelationUpdateSchema>

export const userRoleRelationSchemas = {
  model: userRoleRelationModelSchema,
  dto: userRoleRelationDtoSchema,
  input: userRoleRelationInputSchema,
  update: userRoleRelationUpdateSchema
}

export default userRoleRelationTable