import { Service } from "typedi";
import type { AchievementInput, AchievementModel, AchievementUpdate } from "../schema/achievement.schema";
import db from "..";
import achievementTable, { achievementSchemas } from "../schema/achievement.schema";
import { eq } from "drizzle-orm";

@Service()
export class AchievementRepo {
  async create(achievement: AchievementInput): Promise<AchievementModel> {
    const [ret] = await db
      .insert(achievementTable)
      .values(achievement)
      .returning()

    return achievementSchemas.achievementModelSchema.parse(ret)
  }

  async update(achievement: AchievementUpdate): Promise<AchievementModel> {
    achievement.updated_at = new Date().toISOString()

    const [ret] = await db
      .update(achievementTable)
      .set(achievement)
      .where(eq(achievementTable.id, achievement.id))
      .returning()

    return achievementSchemas.achievementModelSchema.parse(ret)
  }

  async delete(id: AchievementModel['id']): Promise<AchievementModel> {
    const [ret] = await db
      .delete(achievementTable)
      .where(eq(achievementTable.id, id))
      .returning()

    return achievementSchemas.achievementModelSchema.parse(ret)
  }

  async find(
    id: AchievementModel['id']
  ): Promise<AchievementModel | undefined> {
    const achievement = await db.query.achievementTable.findFirst({
      where: eq(achievementTable.id, id)
    })

    if(!achievement) return undefined

    return achievementSchemas.achievementModelSchema.parse(achievement)
  }

  async findMany(): Promise<AchievementModel[]> {
    return achievementSchemas.achievementModelSchema
      .array()
      .parse(await db.query.achievementTable.findMany())
  }
}