import { Service } from "typedi";
import type {
  CollectionInput,
  CollectionModel,
  CollectionUpdate,
} from '../schema/collection.schema'
import db from "..";
import collectionTable, { collectionSchemas } from "../schema/collection.schema";
import { eq } from 'drizzle-orm'

@Service()
export class CollectionRepo {
  async create(collection: CollectionInput): Promise<CollectionModel> {
    const [ret] = await db
      .insert(collectionTable)
      .values(collection)
      .returning()

    return collectionSchemas.collectionModelSchema.parse(ret)
  }

  async update(collection: CollectionUpdate): Promise<CollectionModel> {

    collection.updated_at = new Date().toISOString();

    const [ret] = await db
      .update(collectionTable)
      .set(collection)
      .where(eq(collectionTable.id, collection.id))
      .returning()

    return collectionSchemas.collectionModelSchema.parse(ret)
  }

  async delete(id: CollectionModel['id']): Promise<CollectionModel> {

    const [ret] = await db
      .update(collectionTable)
      .set({ deleted_at: new Date().toISOString(), is_active: false })
      .where(eq(collectionTable.id, id))
      .returning();

    return collectionSchemas.collectionModelSchema.parse(ret)
  }

  async deletePermanently(id: CollectionModel['id']): Promise<CollectionModel> {
    const [ret] = await db
      .delete(collectionTable)
      .where(eq(collectionTable.id, id))
      .returning()

    return collectionSchemas.collectionModelSchema.parse(ret)
  }

  async restore(id: CollectionModel['id']): Promise<CollectionModel> {
    const [ret] = await db
      .update(collectionTable)
      .set({ deleted_at: null, is_active: true })
      .where(eq(collectionTable.id, id))
      .returning()

    return collectionSchemas.collectionModelSchema.parse(ret)
  }

  async find(
    id: CollectionModel['id']
  ): Promise<CollectionModel | undefined> {
    const collection = await db.query.collectionTable.findFirst({
      where: eq(collectionTable.id, id),
    })

    if (collection?.deleted_at === null) 
      return collectionSchemas.collectionModelSchema.parse(collection)
  }

  async findMany(): Promise<CollectionModel[]> {
    return collectionSchemas.collectionModelSchema
      .array()
      .parse(await db.query.collectionTable.findMany())
  }
}