import { Service } from "typedi";
import db from "..";
import type { LanguageInput, LanguageModel, LanguageUpdate } from "../schema/language.schema";
import languageTable, { languageSchema } from "../schema/language.schema";
import { eq } from "drizzle-orm";


@Service()
export class LanguageRepo {
    async create(language: LanguageInput): Promise<LanguageModel> {
        const [ret] = await db
            .insert(languageTable)
            .values(language)
            .returning()

        return languageSchema.model.parse(ret)
    }

    async update(language: LanguageUpdate): Promise<LanguageModel> {
        const [ret] = await db
            .update(languageTable)
            .set(language)
            .where(eq(languageTable.id, language.id))
            .returning()

        return languageSchema.model.parse(ret)
    }

    async delete(id: LanguageModel['id']): Promise<LanguageModel> {
        const [ret] = await db
            .delete(languageTable)
            .where(eq(languageTable.id, id))
            .returning()

        return languageSchema.model.parse(ret)
    }

    async find(id: LanguageModel['id']): Promise<LanguageModel | undefined> {
        const language = await db.query.languageTable.findFirst({
            where: eq(languageTable.id, id),
        })

        return languageSchema.model.parse(language)
    }

    async findMany(): Promise<LanguageModel[]> {
        const languages = await db.query.languageTable.findMany()
        // o codigo fica mais legivel com map, apesar disso estamos usando outra forma em stats-resources.repo.ts
        return languages.map((language) => languageSchema.model.parse(language))
    }
}