import { Service } from "typedi";
import type { LicenseInput, LicenseModel, LicenseUpdate } from "../schema/license.schema";
import db from "..";
import licenseTable, { licenseSchema } from "../schema/license.schema";
import { eq } from "drizzle-orm";


@Service()
export class LicenseRepo {
    async create(license: LicenseInput): Promise<LicenseModel> {
        const [ret] = await db
            .insert(licenseTable)
            .values(license)
            .returning()

        return licenseSchema.model.parse(ret)
    }

    async update(license: LicenseUpdate): Promise<LicenseModel> {
        const [ret] = await db
            .update(licenseTable)
            .set(license)
            .where(eq(licenseTable.id, license.id))
            .returning()

        return licenseSchema.model.parse(ret)
    }

    async delete(id: LicenseModel['id']): Promise<LicenseModel> {
        const [ret] = await db
            .delete(licenseTable)
            .where(eq(licenseTable.id, id))
            .returning()

        return licenseSchema.model.parse(ret)
    }

    async find(id: LicenseModel['id']): Promise<LicenseModel | undefined> {
        const license = await db.query.licenseTable.findFirst({
            where: eq(licenseTable.id, id)
        })

        return licenseSchema.model.parse(license);
    }

    async findMany(): Promise<LicenseModel[]> {
        return licenseSchema.model.array().parse(await db.query.licenseTable.findMany())
    }
}
