import { Service } from "typedi";
import type { ObjectTypeInput, ObjectTypeModel, ObjectTypeUpdate } from "../schema/object-type.schema";
import db from "..";
import objectTypeTable, { objectTypeSchema } from "../schema/object-type.schema";
import { eq } from "drizzle-orm";


@Service()
export class objectTypeRepo {
    async create(objectType: ObjectTypeInput): Promise<ObjectTypeModel> {
        const [ret] = await db
            .insert(objectTypeTable)
            .values(objectType)
            .returning()
        return objectTypeSchema.model.parse(ret)
    }

    async update(objectType: ObjectTypeUpdate): Promise<ObjectTypeModel> {
        const [ret] = await db
            .update(objectTypeTable)
            .set(objectType)
            .where(eq(objectTypeTable.id, objectType.id))
            .returning()
        return objectTypeSchema.model.parse(ret)
    }

    async delete(id: ObjectTypeModel['id']): Promise<ObjectTypeModel> {
        const [ret] = await db
            .delete(objectTypeTable)
            .where(eq(objectTypeTable.id, id))
            .returning()
        return objectTypeSchema.model.parse(ret)
    }

    async find(id: ObjectTypeModel['id']): Promise<ObjectTypeModel | undefined> {
        const objectType = await db.query.objectTypeTable.findFirst({
            where: eq(objectTypeTable.id, id),
        })
        return objectTypeSchema.model.parse(objectType)
    }

    async findMany(): Promise<ObjectTypeModel[]> {
        const objectTypes = await db.query.objectTypeTable.findMany()
        // o codigo fica mais legivel com map, apesar disso estamos usando outra forma em stats-resources.repo.ts
        return objectTypes.map((objectType) => objectTypeSchema.model.parse(objectType))
    }

}