import { Service } from "typedi";
import db from '..'
import resourceStatsTable, { resourceStatsSchema, type ResourceStatsInput, type ResourceStatsModel, type ResourceStatsUpdate } from "../schema/resource-stats.schema";
import { eq } from 'drizzle-orm'
import { sql } from "drizzle-orm";

@Service()
export class statsResourceRepo {
    async create(): Promise<ResourceStatsModel> {
        const statsResource: ResourceStatsInput = {}
        const [ret] = await db
            .insert(resourceStatsTable)
            .values(statsResource)
            .returning()

        return resourceStatsSchema.model.parse(ret)
    }

    async update(statsResource: ResourceStatsUpdate): Promise<ResourceStatsModel> {
        const [ret] = await db
            .update(resourceStatsTable)
            .set(statsResource)
            .where(eq(resourceStatsTable.id, statsResource.id))
            .returning()

        return resourceStatsSchema.model.parse(ret)
    }

    async sharesUpdate(id: ResourceStatsModel['id']): Promise<ResourceStatsModel> {
        const [ret] = await db
        .update(resourceStatsTable)
        .set({
            shares: sql`${resourceStatsTable.shares} + 1` // Incrementa diretamente no SQL
        })
        .where(eq(resourceStatsTable.id, id))
        .returning();
        return resourceStatsSchema.model.parse(ret)
    }


    async viewUpdate(id: ResourceStatsModel['id']): Promise<ResourceStatsModel> {
        const [ret] = await db
            .update(resourceStatsTable)
            .set({
                views: sql`${resourceStatsTable.views} + 1` // Incrementa diretamente no SQL
            })
            .where(eq(resourceStatsTable.id, id))
            .returning();
        return resourceStatsSchema.model.parse(ret)
    }

    async downloadUpdate(id: ResourceStatsModel['id']): Promise<ResourceStatsModel> {
        const [ret] = await db
            .update(resourceStatsTable)
            .set({
                downloads: sql`${resourceStatsTable.downloads} + 1` // Incrementa diretamente no SQL
            })
            .where(eq(resourceStatsTable.id, id))
            .returning();
        return resourceStatsSchema.model.parse(ret)
    }


    async delete(id: ResourceStatsModel['id']): Promise<ResourceStatsModel> {
        const [ret] = await db
            .delete(resourceStatsTable)
            .where(eq(resourceStatsTable.id, id))
            .returning()

        return resourceStatsSchema.model.parse(ret)
    }

    async find(id: ResourceStatsModel['id']): Promise<ResourceStatsModel | undefined> {
        const statsResource = await db.query.resourceStatsTable.findFirst({
            where: eq(resourceStatsTable.id, id),
        })

        return resourceStatsSchema.model.parse(statsResource)
    }

    async findMany(): Promise<ResourceStatsModel[]> {

        return resourceStatsSchema.model.array().parse(await db.query.resourceStatsTable.findMany())
    }

}