import { Service } from "typedi";
import { resourceSchema, type ResourceInput, type ResourceModel, type ResourceUpdate } from "../schema/resource.schema";
import db from '..'
import resourceTable from "../schema/resource.schema";
import { eq, sql, and } from 'drizzle-orm'


@Service()
export class ResourceRepo {
    async create(resource: ResourceInput
    ): Promise<ResourceModel> {
        const [ret] = await db
            .insert(resourceTable)
            .values(resource)
            .returning()

        return resourceSchema.model.parse(ret)

    }

    async update(resource: ResourceUpdate): Promise<ResourceModel> {
        const [ret] = await db
            .update(resourceTable)
            .set(resource)
            .where(eq(resourceTable.id, resource.id))
            .returning()

        return resourceSchema.model.parse(ret)
    }

    async deleteData(id: ResourceModel['id']): Promise<ResourceModel> {
        const [ret] = await db
            .delete(resourceTable)
            .where(eq(resourceTable.id, id))
            .returning()

        return resourceSchema.model.parse(ret)
    }

    async delete(id: ResourceModel['id']): Promise<ResourceModel> {
        const [ret] = await db
            .update(resourceTable)
            .set({
                deleted_at: sql`NOW()`,
                active: false
            })
            .where(eq(resourceTable.id, id))
            .returning()

        return resourceSchema.model.parse(ret)
    }

    async active(id: ResourceModel['id']): Promise<ResourceModel> {
        const [ret] = await db
            .update(resourceTable)
            .set({
                deleted_at: null,
                active: true
            })
            .where(eq(resourceTable.id, id))
            .returning()

        return resourceSchema.model.parse(ret)
    }


    async find(id: ResourceModel['id']): Promise<ResourceModel | undefined> {
        const resource = await db.query.resourceTable.findFirst({
            where: and(eq(resourceTable.id, id), eq(resourceTable.active, true)),
        })

        return resource ? resourceSchema.model.parse(resource) : undefined;
    }

    async findMany(): Promise<ResourceModel[]> {

        return resourceSchema.model.array().parse(await db.query.resourceTable.findMany({ where: eq(resourceTable.active, true) }))
    }


}