import { Service } from "typedi";
import type { RoleInput, RoleModel, RoleUpdate } from "../schema/role.schema";
import db from "..";
import roleTable, { roleSchemas } from "../schema/role.schema";
import { eq } from "drizzle-orm";
import { z } from "zod";

@Service()
export class RoleRepo {
  async findMany():Promise<RoleModel[]>{
  
    return z.array(roleSchemas.model).parse(await db.query.roleTable.findMany())
  }
  async findById(id: RoleModel['id']): Promise<RoleModel | null>{
    const role = await db.query.roleTable.findFirst({
      where: eq(roleTable.id, id)
    })

    if(!role) return null
    return roleSchemas.model.parse(role)
  }
  async findByName(name: RoleModel['name']): Promise<RoleModel | null>{
    const role = await db.query.roleTable.findFirst({
      where: eq(roleTable.name, name)
    })

    if(!role) return null
    return roleSchemas.model.parse(role)
  }
  async create(role: RoleInput, tx?: db): Promise<RoleModel>{
    const repo = tx ?? db
    const [ret] = await repo
    .insert(roleTable)
    .values(role)
    .returning()

    return roleSchemas.model.parse(ret)
  }
  async update(role: RoleUpdate): Promise<RoleModel>{
    const [ret] = await db
    .update(roleTable)
    .set(role)
    .where(eq(roleTable.id, role.id))
    .returning()

    return roleSchemas.model.parse(ret)
  }
  async delete(id: RoleModel['id']): Promise<RoleModel>{
    const [ret] = await db
    .delete(roleTable)
    .where(eq(roleTable.id, id))
    .returning()

    return roleSchemas.model.parse(ret)
  }
}