import { Service } from "typedi";
import type { SubmissionInput, SubmissionModel, SubmissionUpdate } from "../schema/submission.schema";
import submissionTable, { submissionSchemas } from "../schema/submission.schema";
import db from "..";
import { eq } from "drizzle-orm";

@Service()
export class SubmissionRepo {
  async create(submission: SubmissionInput): Promise<SubmissionModel> {
    const [ret] = await db
      .insert(submissionTable)
      .values(submission)
      .returning()

    return submissionSchemas.submissionModelSchema.parse(ret)
  }

  async update(submission: SubmissionUpdate): Promise<SubmissionModel> {
    submission.updated_at = new Date()

    const[ret] = await db 
      .update(submissionTable)
      .set(submission)
      .where(eq(submissionTable.id, submission.id))
      .returning()

    return submissionSchemas.submissionModelSchema.parse(ret)
  }

  async answer(submission: SubmissionUpdate): Promise<SubmissionModel> {
    submission.answered_at = new Date()

    const [ret] = await db
      .update(submissionTable)
      .set(submission)
      .where(eq(submissionTable.id, submission.id))
      .returning()

    return submissionSchemas.submissionModelSchema.parse(ret)
  }

  async delete(id: SubmissionModel['id']): Promise<SubmissionModel> {
    const [ret] = await db
      .delete(submissionTable)
      .where(eq(submissionTable.id, id))
      .returning()
    
      return submissionSchemas.submissionModelSchema.parse(ret)
  }

  async find(id: SubmissionModel['id']): Promise<SubmissionModel | undefined> {
    const submission = await db.query.submissionTable.findFirst({
      where: eq(submissionTable.id, id),
    })

    return submission ? submissionSchemas.submissionModelSchema.parse(submission) : undefined
  }

  async findMany(): Promise<SubmissionModel[]> {
    return submissionSchemas.submissionModelSchema
    .array()
    .parse(await db.query.submissionTable.findMany())
  }
}