import { eq } from "drizzle-orm";
import { Service } from "typedi";
import db from "@/db";
import userStatsTable, { 
  userStatsSchemas,
  type UserStatsInput,
  type UserStatsModel, 
  type UserStatsUpdate
} from "../schema/user-stats.schema";


@Service()
export class UserStatsRepo {
    async findById(
        id: UserStatsModel['id']
    ): Promise<UserStatsModel | null> {
        const userStats = await db.query.userStatsTable.findFirst({
            where: eq(userStatsTable.id, id)
        })
        if(!userStats) return null
        return userStatsSchemas.model.parse(userStats)
    }

    async create(tx?: db): Promise<UserStatsModel> {
      const repo = tx ?? db
      const userStats: UserStatsInput = {}
      const [ret] = await repo
        .insert(userStatsTable)
        .values(userStats)
        .returning()

      return userStatsSchemas.model.parse(ret)
    }

    async update(userStats: UserStatsUpdate): Promise<UserStatsModel>{
      const [ret] = await db
      .update(userStatsTable)
      .set(userStats)
      .where(eq(userStatsTable.id, userStats.id))
      .returning()

      return userStatsSchemas.model.parse(ret)
    }

    async delete(id: UserStatsModel['id']): Promise<UserStatsModel>{
      const [ret] = await db
      .delete(userStatsTable)
      .where(eq(userStatsTable.id, id))
      .returning()

      return userStatsSchemas.model.parse(ret)
    }
}