import { sql } from "drizzle-orm";
import { boolean, integer, numeric, pgTable, serial, text, timestamp, varchar } from "drizzle-orm/pg-core";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import { z } from "zod";
import stateEnum from "./stateEnum.schema";

export const achievementTable = pgTable('achievements', {
  id: serial('id').primaryKey()
    .notNull()
    .unique(),
  name: varchar('name', { length: 255 })
    .notNull(),
  description: text('description'),
  reward_experience: numeric('reward_experience'),
  reward_points: numeric('reward_points'),
  state: stateEnum('state')
    .notNull()
    .default('inactive'),
  repeatable: integer('repeatable')
    .notNull(),
  is_resettable: boolean('is_resettable')
    .notNull(),
  created_at: timestamp('created_at', { mode: 'string' })
    .notNull()
    .defaultNow(),
  updated_at: timestamp('updated_at', { mode: 'string' })
    .notNull()
    .defaultNow()
    .$onUpdate(() => sql`current_timestamp`)
})

const achievementModelSchema = createSelectSchema(achievementTable)
const achievementDtoSchema = achievementModelSchema
const achievementInputSchema = createInsertSchema(achievementTable)
const achievementUpdateSchema = achievementInputSchema.partial().required({ id: true })

export type AchievementModel = z.infer<typeof achievementModelSchema>
export type AchievementDto = z.infer<typeof achievementDtoSchema>
export type AchievementInput = z.infer<typeof achievementInputSchema>
export type AchievementUpdate = z.infer<typeof achievementUpdateSchema>

export const achievementSchemas = {
    achievementModelSchema,
    achievementDtoSchema,
    achievementInputSchema,
    achievementUpdateSchema,
}

export default achievementTable