import { sql } from "drizzle-orm"
import { pgTable, serial, timestamp, varchar } from "drizzle-orm/pg-core"
import { createInsertSchema, createSelectSchema } from "drizzle-zod"
import type { z } from "zod"

const actionTable = pgTable('actions', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  name: varchar('name', {length: 255})
    .notNull(),
  createdAt: timestamp('created_at', { mode: 'string' })
    .notNull()
    .defaultNow(),
  updatedAt: timestamp('updated_at', { mode: 'string' })
    .notNull()
    .defaultNow()
    .$onUpdate(() => sql`current_timestamp`),  
})

const actionModelSchema = createSelectSchema(actionTable)
const actionDtoSchema = actionModelSchema
const actionInputSchema = createInsertSchema(actionTable)
const actionUpdateSchema = actionInputSchema
  .partial()
  .required({ id: true })

export type ActionModel = z.infer<typeof actionModelSchema>
export type ActionDto = z.infer<typeof actionDtoSchema>
export type ActionInput = z.infer<typeof actionInputSchema>
export type ActionUpdate = z.infer<typeof actionUpdateSchema>

export const actionSchemas = {
  model: actionModelSchema,
  dto: actionDtoSchema,
  input: actionInputSchema,
  update: actionUpdateSchema
}

export default actionTable