import {
    pgTable,
    serial, 
    integer,
    numeric
  } from 'drizzle-orm/pg-core'
  import { createInsertSchema, createSelectSchema } from 'drizzle-zod'
  import { z } from 'zod'

  const collectionStatsTable = pgTable('collection_stats', {
    id: serial('id').primaryKey()
      .notNull()
      .unique(),
    views: integer('views')
      .notNull()
      .default(0),
    downloads: integer('downloads')
      .notNull()
      .default(0),
    likes: integer('likes')
      .notNull()
      .default(0),
    shares: integer('shares')
      .notNull()
      .default(0),
    score: numeric('score')
      .notNull()
      .default('0.0'),
    follows: integer('follows')
      .notNull()
      .default(0),
  })

  const collectionStatsModelSchema = createSelectSchema(collectionStatsTable)
  const collectionStatsDtoSchema = collectionStatsModelSchema
  const collectionStatsInputSchema = createInsertSchema(collectionStatsTable)
  const collectionStatsUpdateSchema = collectionStatsInputSchema.partial().required({ id: true })

export type CollectionStatsModel = z.infer<typeof collectionStatsModelSchema>
export type CollectionStatsDto = z.infer<typeof collectionStatsDtoSchema>
export type CollectionStatsInput = z.infer<typeof collectionStatsInputSchema>
export type CollectionStatsUpdate = z.infer<typeof collectionStatsUpdateSchema>

export const collectionStatsSchemas = {
  collectionStatsModelSchema,
  collectionStatsDtoSchema,
  collectionStatsInputSchema,
  collectionStatsUpdateSchema,
}

export default collectionStatsTable