import { relations } from "drizzle-orm";
import { pgTable, serial, varchar } from "drizzle-orm/pg-core";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from 'zod'


const educationalStageTable = pgTable('educational_stages', {
    id: serial('id').primaryKey()
      .unique()
      .notNull(),
    name: varchar('name', { length: 255 })
      .notNull(),
})

export const educationalStagesRelations = relations(educationalStageTable, ({many}) => ({
    resourceEducationalStages: many(educationalStageTable) 
}))

const educationalStagesModelSchema = createSelectSchema(educationalStageTable)
const educationalStagesDtoSchema = educationalStagesModelSchema.omit({})
const educationalStagesInputSchema = createInsertSchema(educationalStageTable)
const educationalStagesUpdateSchema = educationalStagesInputSchema
    .partial()
    .required({ id: true })

export type EducationalStagesModel = z.infer<typeof educationalStagesModelSchema>
export type EducationalStagesDto = z.infer<typeof educationalStagesDtoSchema>
export type EducationalStagesInput = z.infer<typeof educationalStagesInputSchema>
export type EducationalStagesUpdate = z.infer<typeof educationalStagesUpdateSchema>

export const educationalStageSchema = {
    model: educationalStagesModelSchema,
    dto: educationalStagesDtoSchema,
    input: educationalStagesInputSchema,
    update: educationalStagesUpdateSchema,
}

export default educationalStageTable