import { relations } from "drizzle-orm";
import { pgTable, serial,varchar } from "drizzle-orm/pg-core";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from 'zod'
import resourceLanguagesTable from "../relations/resource-language.relation";


const languageTable = pgTable('languages', {
    id: serial('id').primaryKey()
      .unique()
      .notNull(),
    name: varchar('name', { length: 255 })
      .notNull(),
    code: varchar('code', { length: 10 })
      .notNull()
      .unique()
})


export const languageRelations = relations(languageTable, ({many}) => ({
    resourceLanguages: many(resourceLanguagesTable) 
})
)

const languageModelSchema = createSelectSchema(languageTable)
const languageDtoSchema = languageModelSchema.omit({})
const languageInputSchema = createInsertSchema(languageTable)
const languageUpdateSchema = languageInputSchema
    .partial()
    .required({ id: true })

export type LanguageModel = z.infer<typeof languageModelSchema>
export type LanguageDto = z.infer<typeof languageDtoSchema>
export type LanguageInput = z.infer<typeof languageInputSchema>
export type LanguageUpdate = z.infer<typeof languageUpdateSchema>

export const languageSchema = {
    model: languageModelSchema,
    dto: languageDtoSchema,
    input: languageInputSchema,
    update: languageUpdateSchema,
}

export default languageTable