import { serial, varchar, pgTable } from "drizzle-orm/pg-core";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from 'zod'


const objectTypeTable = pgTable("object_types", {
    id: serial("id").primaryKey()
      .notNull()
      .unique(),
    name: varchar("name", { length: 255 })
      .notNull(),
}
)

const objectTypeModelSchema = createSelectSchema(objectTypeTable)
const objectTypeDtoSchema = objectTypeModelSchema.omit({})
const objectTypeInputSchema = createInsertSchema(objectTypeTable)
const objectTypeUpdateSchema = objectTypeInputSchema
    .partial()
    .required({ id: true })

export type ObjectTypeModel = z.infer<typeof objectTypeModelSchema>
export type ObjectTypeDto = z.infer<typeof objectTypeDtoSchema>
export type ObjectTypeInput = z.infer<typeof objectTypeInputSchema>
export type ObjectTypeUpdate = z.infer<typeof objectTypeUpdateSchema>

export const objectTypeSchema = {
    model: objectTypeModelSchema,
    dto: objectTypeDtoSchema,
    input: objectTypeInputSchema,
    update: objectTypeUpdateSchema,
}

export default objectTypeTable