import {
    pgTable,
    serial,
    integer,
    numeric
} from 'drizzle-orm/pg-core'
import { createInsertSchema, createSelectSchema } from 'drizzle-zod'
import type { z } from 'zod'


const resourceStatsTable = pgTable('resource_stats', {
    id: serial('id').notNull()
      .primaryKey()
      .unique(),
    views: integer('views')
      .notNull()
      .default(0),
    downloads: integer('downloads')
      .notNull()
      .default(0),
    likes: integer('likes')
      .notNull()
      .default(0),
    shares: integer('shares')
      .notNull()
      .default(0),
    score: numeric('score')
      .notNull()
      .default('0.0'),
    follows: integer('follows')
      .notNull()
      .default(0),
})

const resourceStatsModelSchema = createSelectSchema(resourceStatsTable)
const resourceStatsDtoSchema = resourceStatsModelSchema.omit({})
const resourceStatsInputSchema = createInsertSchema(resourceStatsTable)
const resourceStatsUpdateSchema = resourceStatsInputSchema
    .partial()
    .required({ id: true })

export type ResourceStatsModel = z.infer<typeof resourceStatsModelSchema>
export type ResourceStatsDto = z.infer<typeof resourceStatsDtoSchema>
export type ResourceStatsInput = z.infer<typeof resourceStatsInputSchema>
export type ResourceStatsUpdate = z.infer<typeof resourceStatsUpdateSchema>

export const resourceStatsSchema = {
    model: resourceStatsModelSchema,
    dto: resourceStatsDtoSchema,
    input: resourceStatsInputSchema,
    update: resourceStatsUpdateSchema,
}

export default resourceStatsTable
