import { sql } from "drizzle-orm"
import { pgTable, serial, text, timestamp, varchar } from "drizzle-orm/pg-core"
import { createInsertSchema, createSelectSchema } from "drizzle-zod"
import type { z } from "zod"

const roleTable = pgTable('roles', {
  id: serial('id').primaryKey()
    .unique()
    .notNull(),
  name: varchar('name', {length: 255})
    .unique()
    .notNull(),
  description: text('description'),
  createdAt: timestamp('created_at', { mode: 'string' })
    .notNull()
    .defaultNow(),
  updatedAt: timestamp('updated_at', { mode: 'string' })
    .notNull()
    .defaultNow()
    .$onUpdate(() => sql`current_timestamp`),  
})

const roleModelSchema = createSelectSchema(roleTable)
const roleDtoSchema = roleModelSchema
const roleInputSchema = createInsertSchema(roleTable)
const roleUpdateSchema = roleInputSchema
  .partial()
  .required({ id: true })

export type RoleModel = z.infer<typeof roleModelSchema>
export type RoleDto = z.infer<typeof roleDtoSchema>
export type RoleInput = z.infer<typeof roleInputSchema>
export type RoleUpdate = z.infer<typeof roleUpdateSchema>

export const roleSchemas = {
  model: roleModelSchema,
  dto: roleDtoSchema,
  input: roleInputSchema,
  update: roleUpdateSchema
}

export default roleTable