import {
    pgTable,
    serial,
    varchar,
} from 'drizzle-orm/pg-core'
import { createInsertSchema, createSelectSchema } from 'drizzle-zod'
import type { z } from 'zod'




const subjectsTable = pgTable('subjects', {
    id: serial('id').primaryKey()
      .unique()
      .notNull(),
    name: varchar('name', { length: 255 })
      .notNull(),
})

const subjectModelSchema = createSelectSchema(subjectsTable)
const subjectDtoSchema = subjectModelSchema.omit({})
const subjectInputSchema = createInsertSchema(subjectsTable)
const subjectUpdateSchema = subjectInputSchema
    .partial()
    .required({ id: true })

export type SubjectModel = z.infer<typeof subjectModelSchema>
export type SubjectDto = z.infer<typeof subjectDtoSchema>
export type SubjectInput = z.infer<typeof subjectInputSchema>
export type SubjectUpdate = z.infer<typeof subjectUpdateSchema>

export const subjectSchema = {
    model: subjectModelSchema,
    dto: subjectDtoSchema,
    input: subjectInputSchema,
    update: subjectUpdateSchema,
}

export default subjectsTable