import { integer, numeric, pgTable, serial } from "drizzle-orm/pg-core";
import { createInsertSchema, createSelectSchema } from "drizzle-zod";
import type { z } from "zod";

const userStatsTable = pgTable('user_stats', {
  id: serial('id').primaryKey()
  .unique()
  .notNull(),
  score: numeric('score')
  .notNull()
  .default('0.0'),
  likes: integer('likes').notNull()
  .default(0),
  likes_received: integer('likes_received').notNull()
  .default(0),
  follows: integer('follows').notNull()
  .default(0),
  followers: integer('followers').notNull()
  .default(0),
  collections: integer('collections').notNull()
  .default(0),
  submitted_resources: integer('submitted_resources').notNull()
  .default(0),
  approved_resources: integer('approved_resources').notNull()
  .default(0),
  reviewed_resources: integer('reviewed_resources').notNull()
  .default(0),
  comments: integer('comments').notNull()
  .default(0),
})

const userStatsModelSchema = createSelectSchema(userStatsTable)
const userStatsDtoSchema = userStatsModelSchema
const userStatsInputSchema = createInsertSchema(userStatsTable)
const userStatsUpdateSchema = userStatsInputSchema
  .partial()
  .required({ id: true})

export type UserStatsModel = z.infer<typeof userStatsModelSchema>
export type UserStatsDto = z.infer<typeof userStatsDtoSchema>
export type UserStatsInput = z.infer<typeof userStatsInputSchema>
export type UserStatsUpdate = z.infer<typeof userStatsUpdateSchema>

export const userStatsSchemas = {
  model: userStatsModelSchema,
  dto: userStatsDtoSchema,
  input: userStatsInputSchema,
  update: userStatsUpdateSchema
}

export default userStatsTable