import type db from "@/db";
import { 
  type FollowRelationInput,
  type FollowRelationModel, 
  type FollowRelationUpdate 

  } from "@/db/relations/followers.relation";
import { FollowRepo } from "@/db/repo/follow.repo";
import { Inject, Service } from "typedi";

@Service()
export class FollowRelationService {
  
  @Inject()
  private readonly repo: FollowRepo

  async findMany(): Promise<FollowRelationModel[]> {
    return await this.repo.findMany()
  }

  async findByFollow(followRelation: FollowRelationInput): Promise<number> {
    return await this.repo.findByFollow(followRelation)
      
  }

  async findById(
    id:  FollowRelationModel['id']
  ): Promise<FollowRelationModel | null>{
    return await this.repo.findById(id)
  }

  async findByUserId(
    userId:  FollowRelationModel['user_id']
  ): Promise<FollowRelationModel[]>{
    return await this.repo.findByUserId(userId)
  }

  async findByFollowerId(
    followerId:  FollowRelationModel['follower_id']
  ): Promise<FollowRelationModel[]>{
    return await this.repo.findByFollowerId(followerId)
  }

  async create(followRelation: FollowRelationInput, tx?: db): Promise<FollowRelationModel> {

    return this.repo.create(followRelation, tx)
  }

  async update(followRelation: FollowRelationUpdate): Promise<FollowRelationModel> {
    
    return this.repo.update(followRelation)
  }


  async delete(user: FollowRelationModel['id']): Promise<FollowRelationModel> {
    
    return this.repo.delete(user)
  }

}