import { userAchievementsRepo } from "@/db/repo/user-achievements.repo";
import type { AchievementModel } from "@/db/schema/achievement.schema";
import type { UserModel } from "@/db/schema/user.schema";
import { Inject, Service } from "typedi";

@Service()
export class UserAchievementsService {
    @Inject()
    private readonly repo: userAchievementsRepo

    async associateUserWithAchievements(userId: UserModel['id'], achievementIds: AchievementModel['id'][]): Promise<void> {
        await this.repo.associateUserWithAchievements(userId, achievementIds);
    }

    async getAchievementsByUser(userId: UserModel['id']): Promise<AchievementModel[]> {
        return this.repo.getAchievementsByUser(userId);
    }

    async removeAchievementsFromUser(userId: UserModel['id'], achievementIds: AchievementModel['id'][]): Promise<void> {
        await this.repo.removeAchievementsFromUser(userId, achievementIds);
    }

    async isAssociationExists(userId: UserModel['id'], achievementId: AchievementModel['id']): Promise<boolean> {
        return this.repo.isAssociationExists(userId, achievementId);
    }

    async getUsersByAchievement(achievementId: AchievementModel['id']): Promise<UserModel[]> {
        return this.repo.getUsersByAchievement(achievementId);
    }

    async updateUserAchievements(userId: UserModel['id'], achievementIds: AchievementModel['id'][]): Promise<void> {
        await this.repo.updateUserAchievements(userId, achievementIds);
    }
}