import type db from "@/db";
import { 
  type UserRoleRelationInput,
  type UserRoleRelationModel, 
  type UserRoleRelationUpdate 
  } from "@/db/relations/user-role.relation";
import { UserRoleRepo } from "@/db/repo/user-role.repo";
import { Inject, Service } from "typedi";

@Service()
export class UserRoleRelationService {
  
  @Inject()
  private readonly repo: UserRoleRepo

  async findMany(): Promise<UserRoleRelationModel[]> {

    return await this.repo.findMany()
  }

  async findByUserRole(userRoleRelation: UserRoleRelationInput): Promise<number> {
    
    return await this.repo.findByUserRole(userRoleRelation)
  }

  async findById(
    id:  UserRoleRelationModel['id']
  ): Promise<UserRoleRelationModel | null>{
    
    return await this.repo.findById(id)
  }

  async findByUserId(
    userId:  UserRoleRelationModel['user_id']
  ): Promise<UserRoleRelationModel[]>{
    
    return await this.repo.findByUserId(userId)
  }

  async findByRoleId(
    roleId:  UserRoleRelationModel['role_id']
  ): Promise<UserRoleRelationModel[]>{
    
    return await this.repo.findByRoleId(roleId)
  }

  async create(followRelation: UserRoleRelationInput, tx?: db): Promise<UserRoleRelationModel> {

    return this.repo.create(followRelation, tx)
  }

  async update(followRelation: UserRoleRelationUpdate): Promise<UserRoleRelationModel> {
    
    return this.repo.update(followRelation)
  }


  async delete(user: UserRoleRelationModel['id']): Promise<UserRoleRelationModel> {
    
    return this.repo.delete(user)
  }

}